/*
 * NAME
 *   g15-lcd.h
 * COPYRIGHT
 *   All Code within g15-lcd.h is copyright (c) Terence Sharp
 *  You may derive this work solong you give credit
 *
 *   Code in EZLCD is copyright Logitech and so, Please refer
 *   To their licenses.
 *
 * PURPOSE
 *   Provides management of controls on the LCD along with
 *   Easy to use functions to start and stop the LCD, Add text
 *   And more.
 *
 *  YES, I KNOW IT IS HACKY TO USE A HEADER TO SPLIT CODE!
 *
 * AUTHOR
 *   Terence Sharp (tezzanator@tezzanator.net)
 *
 * CREATION DATE
 *   04/09/2007
 *
 * Yes, I know i should of used voids; I wasn't thinking straight :/
 */

// Logitech LCD Header
#include "EZ_LCD.h"
// Strings
#include "string.h"


CEzLcd* ezlcd_ = new CEzLcd();

////////////////////////
HRESULT     res_ = S_OK;
HANDLE texts[64];
HANDLE bars[64];
HANDLE boxs[128];
HANDLE pixels[128];
bool debugging = false;

int setdebug()
{
	debugging = true;
	return 0;
}

int dprint(char* message)
{
	if(debugging == true)
	{
		Msg(message);
	}
	return 0;
}

//void settextinvert( int id, bool invert )
//{
//	ezlcd_->SetInvert(texts[id],invert);
//}

int setforeground(bool fore)
{
	ezlcd_->SetAsForeground(fore);
	dprint("Foreground set\n");
	return 0;
}

int addtext(int id, bool scrolling, int size, int align, int maxlength, int numlines)
{
	if( id > 63 )
	{
		dprint("ID IS OUT OF RANGE, OBJECT NOT CREATED!\n");
		return 0; 
	}
	// Scrolling?
	LGObjectType LG_TEXT;
	LGTextSize LG_SIZE;
	int LG_ALIGN;

	if (scrolling == true){
	LG_TEXT = LG_SCROLLING_TEXT;	
	}
	if (scrolling == false){
	LG_TEXT = LG_STATIC_TEXT;
	}

	switch( size ){
		case 1 :
		LG_SIZE = LG_SMALL;
		break;
		case 2 :
		LG_SIZE = LG_MEDIUM;
		break;
		case 3 :
		LG_SIZE = LG_BIG;
		break;
	}

	switch( align ){
		case 1 :
		LG_ALIGN = DT_LEFT;
		break;
		case 2 :
		LG_ALIGN = DT_CENTER;
		break;
		case 3 :
		LG_ALIGN = DT_RIGHT;
		break;
	}

	// It gets created
	texts[id] = ezlcd_->AddText(LG_TEXT, LG_SIZE , LG_ALIGN, maxlength, numlines);
	dprint("Standard text was added\n");
	return 0;
}

int addtextfont(int id, bool scrolling, int size, int align, int maxlength, int numlines, const char* font)
{
	if( id > 63 )
	{
		dprint("ID IS OUT OF RANGE, OBJECT NOT CREATED!\n");
		return 0; 
	}
	// Scrolling?
	LGObjectType LG_TEXT;
	LGTextSize LG_SIZE;
	int LG_ALIGN;

	if (scrolling == true){
	LG_TEXT = LG_SCROLLING_TEXT;	
	}
	if (scrolling == false){
	LG_TEXT = LG_STATIC_TEXT;
	}

	switch( size ){
		case 1 :
		LG_SIZE = LG_SMALL;
		break;
		case 2 :
		LG_SIZE = LG_MEDIUM;
		break;
		case 3 :
		LG_SIZE = LG_BIG;
		break;
	}

	switch( align ){
		case 1 :
		LG_ALIGN = DT_LEFT;
		break;
		case 2 :
		LG_ALIGN = DT_CENTER;
		break;
		case 3 :
		LG_ALIGN = DT_RIGHT;
		break;
	}

	// It gets created
	texts[id] = ezlcd_->AddTextfont(LG_TEXT, LG_SIZE , LG_ALIGN, maxlength, numlines, font);
	dprint("Text with a font was added\n");
	return 0;
}

bool istextidempty( int id )
{
	if( id > 63 )
	{
		dprint("Id Out of range, Rturning false!\n");
		return false;
	}
	if( texts[id] == NULL)
	{
		dprint("Text id IS empty\n");
		return true;
	}
	dprint("Text id is NOT empty\n");
	return false;
}

int removetext( int id )
{
	ezlcd_->KillObject(ezlcd_->getobjectbyhandle(texts[id]));
	dprint("Text object was destroyed!\n");
	texts[id] = NULL;
	dprint("Text ID has been freed up\n");
	return 0;
}

int settextorigin(int id, int x, int y)
{
	// Set the position
	ezlcd_->SetOrigin(texts[id], x, y);
	dprint("Moved text origin\n");
	return 0;
}

int settexttext(int id, const char* text )
{
	// Set the text info
	ezlcd_->SetText(texts[id], _T(text));
	dprint("Text text was set!\n");
	return 0;
}

int settextvis ( int id, bool visible )
{
	ezlcd_->SetVisible(texts[id], visible);
	dprint("Text visibility changed!\n");
	return 0;
}

int settextalign( int id, int align)
{
	ezlcd_->SetTextAlignment(texts[id], align);
	dprint("Text alignment changed\n");
	return 0;
}

int addpbar( int id, int type )
{
	if( id > 63 )
	{
		dprint("ID IS OUT OF RANGE, OBJECT NOT CREATED!\n");
		return 0; 
	}
	LGProgressBarType typei;
		switch( type ){
			case 1 :
			typei = LG_CURSOR;
			break;
			case 2 :
			typei = LG_FILLED;
			break;
			case 3 :
			typei = LG_DOT_CURSOR;
			break;
		}
	bars[id] = ezlcd_->AddProgressBar(typei);
	dprint("Progress bar added\n");
	return 0;
}

bool ispbaridempty( int id )
{
	if( id > 63 )
	{
		dprint("Id Out of range, Returning false!\n");
		return false;
	}
	if( bars[id] == NULL)
	{
		dprint("Bar ID IS empty!\n");
		return true;
	}
	dprint("Bar ID is NOT empty!\n");
	return false;
}

int removepbar( int id )
{
	ezlcd_->KillObject(ezlcd_->getobjectbyhandle(bars[id]));
	dprint("Bar object destroyed!\n");
	bars[id] = NULL;
	dprint("Bar ID freed!\n");
	return 0;
}

int setpbarorigin( int id, int x, int y )
{
	ezlcd_->SetOrigin(bars[id], x, y);	
	dprint("Bar origin moved!\n");
	return 0;
}

int setpbarsize( int id, int width, int height)
{
	ezlcd_->SetProgressBarSize(bars[id], width, height);
	dprint("Bar size set!\n");
	return 0;
}

int setpbarvalue( int id, int value )
{
	ezlcd_->SetProgressBarPosition(bars[id], value);
	dprint("Bar value changed\n");
	return 0;
}

int setpbarvis ( int id, bool visible )
{
	ezlcd_->SetVisible(bars[id], visible);
	dprint("Bar visibilty Changed!\n");
	return 0;
}

int addfbox( int id, int x, int y, int width, int height)
{
	if( id > 127 )
	{
		dprint("ID IS OUT OF RANGE, OBJECT NOT CREATED!\n");
		return 0; 
	}
	boxs[id] = ezlcd_->AddProgressBar(LG_FILLED);
	ezlcd_->SetProgressBarSize(boxs[id], width, height);
	ezlcd_->SetOrigin(boxs[id], x, y);	
	ezlcd_->SetProgressBarPosition(boxs[id], 100);
	dprint("A Filled box was added!\n");
	return 0;
}

int addebox( int id, int x, int y, int width, int height)
{
	if( id > 127 )
	{
		dprint("ID IS OUT OF RANGE, OBJECT NOT CREATED!\n");
		return 0; 
	}
	boxs[id] = ezlcd_->AddProgressBar(LG_FILLED);
	ezlcd_->SetProgressBarSize(boxs[id], width, height);
	ezlcd_->SetOrigin(boxs[id], x, y);	
	ezlcd_->SetProgressBarPosition(boxs[id], 0);
	dprint("An unfilled box was added!\n");
	return 0;
}

bool isboxidempty( int id )
{
	if( id > 127 )
	{
		dprint("Id Out of range, Rturning false!\n");
		return false;
	}
	if( boxs[id] == NULL)
	{
		dprint("Box id IS empty!\n");
		return true;
	}
	dprint("Box ID is NOT empty!\n");
	return false;
}

int removebox( int id )
{
	ezlcd_->KillObject(ezlcd_->getobjectbyhandle(boxs[id]));
	dprint("Box object destroyed\n");
	boxs[id] = NULL;
	dprint("Box ID freed up!\n");
	return 0;
}

int setboxorigin( int id, int x, int y )
{
	ezlcd_->SetOrigin(boxs[id], x, y);	
	dprint("Box origin moved\n");
	return 0;
}

int setboxsize( int id, int width, int height )
{
	ezlcd_->SetProgressBarSize(boxs[id], width, height);
	dprint("Changed box size!\n");
	return 0;
}

int setboxfilled( int id, bool filled )
{
	int val;
	if( filled == true )
	{
		val = 100;
	}
	if( filled == false )
	{
		val = 0;
	}
		ezlcd_->SetProgressBarPosition(boxs[id], val);
		dprint("Box Fill changed!\n");
		return 0;
}

int addpx( int id, int x, int y)
{
	if( id > 127 )
	{
		dprint("ID IS OUT OF RANGE, OBJECT NOT CREATED!\n");
		return 0; 
	}
	pixels[id] = ezlcd_->AddProgressBar(LG_FILLED);
	ezlcd_->SetProgressBarSize(pixels[id], 1, 1);
	ezlcd_->SetOrigin(pixels[id], x, y);	
	ezlcd_->SetProgressBarPosition(pixels[id], 100);
	dprint("Added pixel!\n");
	return 0;
}

bool ispxidempty( int id )
{
	if( id > 127 )
	{
		dprint("Id Out of range, Rturning false!\n");
		return false;
	}
	if( pixels[id] == NULL)
	{
		dprint("Pixel id IS empty!\n");
		return true;
	}
	dprint("Pixel id is NOT empty!\n");
	return false;
}

int removepx( int id )
{
	ezlcd_->KillObject(ezlcd_->getobjectbyhandle(pixels[id]));
	dprint("Pixel object destroyed\n");
	pixels[id] = NULL;
	dprint("Pixel ID Freed up\n");
	return 0;
}

int setpxvis ( int id, bool visible )
{
	ezlcd_->SetVisible(pixels[id], visible);
	dprint("Pixel visibility changed\n");
	return 0;
}

int setboxvis ( int id, bool visible )
{
	ezlcd_->SetVisible(boxs[id], visible);
	dprint("Box visibilty changed\n");
	return 0;
}

int setpxorigin( int id, int x, int y )
{
	ezlcd_->SetOrigin(pixels[id], x, y);
	dprint("Pixel position changed!\n");
	return 0;
}







HBITMAP LoadBitmapFile(const TCHAR *filename)
{
	return (HBITMAP)LoadImage(NULL, filename, IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE);
}







int createLCD(bool bEmulator,const char* cName)
{
	// Have it initialize itself
    res_ = ezlcd_->InitYourself(_T(cName));
	dprint("Registered with LCD Manager\n");

    if (res_ != S_OK)
    {
        // Something went wrong, when connecting to the LCD Manager software. We need to get out now
        delete ezlcd_;
		Msg("There was an error creating the LCD, Is LCDManager running?\n");
        return 0;
    }

	// Family to use
	if(bEmulator == false)
	{
		ezlcd_->SetDeviceFamilyToUse(LGLCD_DEVICE_FAMILY_KEYBOARD_G15);
		dprint("Using Real hardware!\n");
	}
	if(bEmulator == true)
	{
		ezlcd_->SetDeviceFamilyToUse(LGLCD_DEVICE_FAMILY_LCDEMULATOR_G15);
		dprint("Using emulator!\n");
	}


	//HANDLE egg = ezlcd_->AddBitmap(32,32);
	//ezlcd_->SetBitmap( egg, LoadBitmapFile("C:\\egg.bmp") );
	//ezlcd_->SetOrigin( egg, 0, 0 );



	dprint("Create LCD Finished\n");
	return 0;
}



int updateLCD(){
	// Update LCD
	ezlcd_->Update();
	return 0;
}

int killLCD(){
	// Close the LCD
	dprint("Closing the LCD\n");
	delete ezlcd_;
	return 0;
}